@@ -87,6 +87,63 @@ where
 /// ```
 pub trait Condition<K> {
     fn matches_object(&self, obj: Option<&K>) -> bool;
+
+    /// Returns a `Condition` that holds if `self` does not
+    ///
+    /// # Usage
+    ///
+    /// ```rust
+    /// # use kube_runtime::wait::Condition;
+    /// let condition: fn(Option<&()>) -> bool = |_| true;
+    /// assert!(condition.matches_object(None));
+    /// assert!(!condition.not().matches_object(None));
+    /// ```
+    fn not(self) -> conditions::Not<Self>
+    where
+        Self: Sized,
+    {
+        conditions::Not(self)
+    }
+
+    /// Returns a `Condition` that holds if `self` and `other` both do
+    ///
+    /// # Usage
+    ///
+    /// ```rust
+    /// # use kube_runtime::wait::Condition;
+    /// let cond_false: fn(Option<&()>) -> bool = |_| false;
+    /// let cond_true: fn(Option<&()>) -> bool = |_| true;
+    /// assert!(!cond_false.and(cond_false).matches_object(None));
+    /// assert!(!cond_false.and(cond_true).matches_object(None));
+    /// assert!(!cond_true.and(cond_false).matches_object(None));
+    /// assert!(cond_true.and(cond_true).matches_object(None));
+    /// ```
+    fn and<Other: Condition<K>>(self, other: Other) -> conditions::And<Self, Other>
+    where
+        Self: Sized,
+    {
+        conditions::And(self, other)
+    }
+
+    /// Returns a `Condition` that holds if either `self` or `other` does
+    ///
+    /// # Usage
+    ///
+    /// ```rust
+    /// # use kube_runtime::wait::Condition;
+    /// let cond_false: fn(Option<&()>) -> bool = |_| false;
+    /// let cond_true: fn(Option<&()>) -> bool = |_| true;
+    /// assert!(!cond_false.or(cond_false).matches_object(None));
+    /// assert!(cond_false.or(cond_true).matches_object(None));
+    /// assert!(cond_true.or(cond_false).matches_object(None));
+    /// assert!(cond_true.or(cond_true).matches_object(None));
+    /// ```
+    fn or<Other: Condition<K>>(self, other: Other) -> conditions::Or<Self, Other>
+    where
+        Self: Sized,
+    {
+        conditions::Or(self, other)
+    }
 }
 
 impl<K, F: Fn(Option<&K>) -> bool> Condition<K> for F {
@@ -134,6 +191,41 @@ pub mod conditions {
             false
         }
     }
+
+    /// See [`Condition::not`]
+    #[derive(Copy, Clone, Debug, PartialEq, Eq)]
+    pub struct Not<A>(pub(super) A);
+    impl<A: Condition<K>, K> Condition<K> for Not<A> {
+        fn matches_object(&self, obj: Option<&K>) -> bool {
+            !self.0.matches_object(obj)
+        }
+    }
+
+    /// See [`Condition::and`]
+    #[derive(Copy, Clone, Debug, PartialEq, Eq)]
+    pub struct And<A, B>(pub(super) A, pub(super) B);
+    impl<A, B, K> Condition<K> for And<A, B>
+    where
+        A: Condition<K>,
+        B: Condition<K>,
+    {
+        fn matches_object(&self, obj: Option<&K>) -> bool {
+            self.0.matches_object(obj) && self.1.matches_object(obj)
+        }
+    }
+
+    /// See [`Condition::or`]
+    #[derive(Copy, Clone, Debug, PartialEq, Eq)]
+    pub struct Or<A, B>(pub(super) A, pub(super) B);
+    impl<A, B, K> Condition<K> for Or<A, B>
+    where
+        A: Condition<K>,
+        B: Condition<K>,
+    {
+        fn matches_object(&self, obj: Option<&K>) -> bool {
+            self.0.matches_object(obj) || self.1.matches_object(obj)
+        }
+    }
 }
 
 /// Utilities for deleting objects
